<?php

/**
 * @author     M2E Pro Developers Team
 * @copyright  M2E LTD
 * @license    Commercial use is forbidden
 */

namespace Ess\M2ePro\Model\Amazon\Template\Description;

/**
 * Class \Ess\M2ePro\Model\Amazon\Template\Description\Specific
 */
class Specific extends \Ess\M2ePro\Model\ActiveRecord\Component\AbstractModel
{
    public const DICTIONARY_TYPE_TEXT = 1;
    public const DICTIONARY_TYPE_SELECT = 2;
    public const DICTIONARY_TYPE_CONTAINER = 3;

    public const DICTIONARY_MODE_RECOMMENDED_VALUE = 'recommended_value';
    public const DICTIONARY_MODE_CUSTOM_VALUE = 'custom_value';
    public const DICTIONARY_MODE_CUSTOM_ATTRIBUTE = 'custom_attribute';
    public const DICTIONARY_MODE_NONE = 'none';

    public const TYPE_INT = 'int';
    public const TYPE_FLOAT = 'float';
    public const TYPE_DATETIME = 'date_time';
    public const TYPE_BOOLEAN = 'boolean';

    /**
     * @var \Ess\M2ePro\Model\Template\Description
     */
    private $descriptionTemplateModel = null;

    /**
     * @var \Ess\M2ePro\Model\Amazon\Template\Description\Specific\Source[]
     */
    private $descriptionSpecificSourceModels = [];
    /** @var \Ess\M2ePro\Model\ActiveRecord\Component\Parent\Amazon\Factory  */
    protected $amazonFactory;

    //########################################

    public function __construct(
        \Ess\M2ePro\Model\ActiveRecord\Component\Parent\Amazon\Factory $amazonFactory,
        \Ess\M2ePro\Model\Factory $modelFactory,
        \Ess\M2ePro\Model\ActiveRecord\Factory $activeRecordFactory,
        \Ess\M2ePro\Helper\Factory $helperFactory,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->amazonFactory = $amazonFactory;
        parent::__construct(
            $modelFactory,
            $activeRecordFactory,
            $helperFactory,
            $context,
            $registry,
            $resource,
            $resourceCollection,
            $data
        );
    }

    //########################################

    public function _construct()
    {
        parent::_construct();
        $this->_init(\Ess\M2ePro\Model\ResourceModel\Amazon\Template\Description\Specific::class);
    }

    //########################################

    public function delete()
    {
        $temp = parent::delete();
        $temp && $this->descriptionTemplateModel = null;
        $temp && $this->descriptionSpecificSourceModels = [];

        return $temp;
    }

    //########################################

    /**
     * @return \Ess\M2ePro\Model\Template\Description
     * @throws \Exception
     */
    public function getDescriptionTemplate()
    {
        if ($this->descriptionTemplateModel === null) {
            $this->descriptionTemplateModel = $this->amazonFactory->getCachedObjectLoaded(
                'Template\Description',
                $this->getTemplateDescriptionId(),
                null,
                ['template']
            );
        }

        return $this->descriptionTemplateModel;
    }

    /**
     * @param \Ess\M2ePro\Model\Template\Description $instance
     */
    public function setDescriptionTemplate(\Ess\M2ePro\Model\Template\Description $instance)
    {
        $this->descriptionTemplateModel = $instance;
    }

    /**
     * @return \Ess\M2ePro\Model\Amazon\Template\Description
     * @throws \Exception
     */
    public function getAmazonDescriptionTemplate()
    {
        return $this->getDescriptionTemplate()->getChildObject();
    }

    //########################################

    /**
     * @param \Ess\M2ePro\Model\Magento\Product $magentoProduct
     *
     * @return \Ess\M2ePro\Model\Amazon\Template\Description\Specific\Source
     */
    public function getSource(\Ess\M2ePro\Model\Magento\Product $magentoProduct)
    {
        $productId = $magentoProduct->getProductId();

        if (!empty($this->descriptionSpecificSourceModels[$productId])) {
            return $this->descriptionSpecificSourceModels[$productId];
        }

        $this->descriptionSpecificSourceModels[$productId] = $this->modelFactory->getObject(
            'Amazon_Template_Description_Specific_Source'
        );
        $this->descriptionSpecificSourceModels[$productId]->setMagentoProduct($magentoProduct);
        $this->descriptionSpecificSourceModels[$productId]->setDescriptionSpecificTemplate($this);

        return $this->descriptionSpecificSourceModels[$productId];
    }

    //########################################

    /**
     * @return int
     */
    public function getTemplateDescriptionId()
    {
        return (int)$this->getData('template_description_id');
    }

    /**
     * @return string
     */
    public function getXpath()
    {
        return trim($this->getData('xpath'), '/');
    }

    public function getMode()
    {
        return $this->getData('mode');
    }

    public function getIsRequired()
    {
        return $this->getData('is_required');
    }

    public function getRecommendedValue()
    {
        return $this->getData('recommended_value');
    }

    public function getCustomValue()
    {
        return $this->getData('custom_value');
    }

    public function getCustomAttribute()
    {
        return $this->getData('custom_attribute');
    }

    public function getType()
    {
        return $this->getData('type');
    }

    /**
     * @return array
     */
    public function getAttributes()
    {
        $value = $this->getData('attributes');

        return is_string($value) ? (array)$this->getHelper('Data')->jsonDecode($value) : [];
    }

    //########################################

    public function isRequired()
    {
        return (bool)$this->getIsRequired();
    }

    //----------------------------------------

    public function isModeNone()
    {
        return $this->getMode() == self::DICTIONARY_MODE_NONE;
    }

    public function isModeCustomValue()
    {
        return $this->getMode() == self::DICTIONARY_MODE_CUSTOM_VALUE;
    }

    public function isModeCustomAttribute()
    {
        return $this->getMode() == self::DICTIONARY_MODE_CUSTOM_ATTRIBUTE;
    }

    public function isModeRecommended()
    {
        return $this->getMode() == self::DICTIONARY_MODE_RECOMMENDED_VALUE;
    }

    //----------------------------------------

    public function isTypeInt()
    {
        return $this->getType() == self::TYPE_INT;
    }

    public function isTypeFloat()
    {
        return $this->getType() == self::TYPE_FLOAT;
    }

    public function isTypeDateTime()
    {
        return $this->getType() == self::TYPE_DATETIME;
    }

    public function isTypeBoolean()
    {
        $value = $this->getData($this->getMode());

        if ($this->getType() == self::TYPE_BOOLEAN || is_bool($value)) {
            return true;
        }

        if (is_string($value) && in_array($value, ['true', 'false'], true)) {
            return true;
        }

        return false;
    }

    //########################################
}
